#include "filtering.h"

Kernel *MakeKernel(char *coefs)
{
  Kernel *K;
  AdjRel *A;
  int xsize,ysize,i;
  float val;

  sscanf(coefs,"%d",&xsize);
  coefs=strchr(coefs,',')+1;
  sscanf(coefs,"%d",&ysize);
  coefs=strchr(coefs,',')+1;

  A = Box(xsize, ysize);
  K = CreateKernel(A);
  for (i=0;i<A->n;i++) {
    sscanf(coefs,"%f",&K->val[i]);
    coefs=strchr(coefs,',')+1;
  }

  /* Put the middle value (corresponding to the origin) at the first
     place to match the vector of coefficients with the vector of
     displacements (adjacency relation) */

  for (i=A->n/2; i > 0; i--){
    val = K->val[i];
    K->val[i]=K->val[i-1];
    K->val[i-1]=val;
  }

  DestroyAdjRel(&A);
  return(K);
}

Kernel *CreateKernel(AdjRel *A)
{
  Kernel *K=NULL;
  int i;
  Pixel max, min;

  max.x = max.y = INT_MIN;
  min.x = min.y = INT_MAX;

  K = (Kernel *) calloc(1,sizeof(Kernel));
  if (K == NULL){
    Error(MSG1,"CreateKernel");
  }
  K->val = AllocFloatArray(A->n);
  K->adj = CreateAdjRel(A->n);

  for (i=0;i<A->n;i++) {
    max.x = MAX(A->dx[i],max.x);
    max.y = MAX(A->dy[i],max.y);
    min.x = MIN(A->dx[i],min.x);
    min.y = MIN(A->dy[i],min.y);
    K->adj->dx[i] = A->dx[i];
    K->adj->dy[i] = A->dy[i];
  }
  
  K->xsize = max.x - min.x + 1;
  K->ysize = max.y - min.y + 1;
  
  return(K);
}

void DestroyKernel(Kernel **K)
{
  Kernel *aux;

  aux = *K;
  if(aux != NULL){
    if (aux->val != NULL)   free(aux->val); 
    DestroyAdjRel(&(aux->adj));
    free(aux);    
    *K = NULL;
  }
}

Kernel *FoldKernel(Kernel *K) /* Reflect kernel around x and y */
{
  Kernel *Kf;
  int i,j;

  Kf = CreateKernel(K->adj);
  for (i=0,j=Kf->adj->n-1; i < Kf->adj->n; i++,j--){
    Kf->val[i]     = K->val[j];
    Kf->adj->dx[i] = -K->adj->dx[j];
    Kf->adj->dy[i] = -K->adj->dy[j];
  }
  return(Kf);
}

Image *LinearFilter(Image *img, Kernel *K) /* Filtering in Space */
{
  Image *cimg;
  Pixel u,v,o;
  float conv;
  int p,i,q;
  Kernel *Kf;
  

  cimg = CreateImage(img->ncols+K->xsize-1,img->nrows+K->ysize-1);
  Kf   = FoldKernel(K);
  o.x=o.y=INT_MIN;
  for (i=0; i < Kf->adj->n; i++){ /* (o.x,o.y) is the origin of the
				     original image with respect to
				     the output image */
    if (Kf->adj->dx[i] > o.x)
      o.x = Kf->adj->dx[i];
    if (Kf->adj->dy[i] > o.y)
      o.y = Kf->adj->dy[i];
  } 

  for (u.y=0; u.y < cimg->nrows; u.y++)
    for (u.x=0; u.x < cimg->ncols; u.x++) {
      conv = 0.0;
      p = u.x + cimg->tbrow[u.y];
      for (i=0; i < Kf->adj->n; i++){
	v.x = u.x + Kf->adj->dx[i]; 
	v.y = u.y + Kf->adj->dy[i]; 
	if (ValidPixel(img,v.x-o.x,v.y-o.y)){
	  q = v.x-o.x + img->tbrow[v.y-o.y];
	  conv += (float)img->val[q]*Kf->val[i];
	}	
      }
      cimg->val[p]=(int)conv;
    }

  DestroyKernel(&Kf);
  return(cimg);
}

/*--------------------- Questes deste laboratrio -----------------------*/

/* Questo 1 (1,0): Calcule a multiplicao entre dois espectros de
   freqncia. */

Spectrum *FilterInFreq(Spectrum *spc1, Spectrum *spc2)
{
  Spectrum *spc3=NULL;

  return(spc3);
}


/* Questo 2 (1,0): Gere o espectro real com ncols x nrows pixels de
   um filtro de Butterworth do tipo passa-baixas freqncias, onde Dl
   indica a freqncia de corte e n o fator de decaimento. Lembre-se
   que o espectro  peridico e pode ser gerado no centro da imagem e
   depois transladado para a origem. */

Spectrum *LowPassFilter(int ncols, int nrows, float Dl, float n)
{
  Spectrum *spc=NULL;

  return(spc);
}


/* Questo 3 (1,0): Gere o espectro real com ncols x nrows pixels de
   um filtro de Butterworth do tipo passa-altas freqncias, onde Dh
   indica a freqncia de corte e n o fator de decaimento. */

Spectrum *HighPassFilter(int ncols, int nrows, float Dh, float n)
{
  Spectrum *spc=NULL;

  return(spc);
}


/* Questo 4 (1,0): Gere o espectro real com ncols x nrows pixels de
   um filtro de Butterworth do tipo passa-faixa, onde Dl e Dh indicam
   as freqncias de corte do passa-baixas e do passa-altas,
   respectivamente, e nl e nh os seus fatores de decaimento. */

Spectrum *BandPassFilter(int ncols, int nrows, float Dl, float nl, float Dh, 
float nh)
{
  Spectrum *spc=NULL;

  return(spc);
}


/* Questo 5 (1,0): Gere o espectro real com ncols x nrows pixels de
   um filtro de Butterworth do tipo rejeita-faixa, onde Dl e Dh
   indicam as freqncias de corte do passa-baixas e do passa-altas,
   respectivamente, e nl e nh os seus fatores de decaimento. */

Spectrum *BandRejectFilter(int ncols, int nrows, float Dl, float nl, float 
Dh, float nh)
{
  Spectrum *spc=NULL;

  return(spc);
}


/* Questo 6 (3,0): Aplique a inversa de Moore-Penrose para transformar
   um espectro com (N+1)x(N+1) pixels em uma mscara com (n+1)x(n+1)
   coeficientes. */

Spectrum *MoorePenrose(int n, int N)  /* Questo 6a (1,5): Gere a
					 matriz inversa generalizada
					 de Moore-Penrose. */ 
{
  Spectrum *spc=NULL;

  return(spc);
}


Kernel *ComputeMask(Spectrum *spc, Spectrum *moore, int n) /* Questo
                                             6b (1,5): Gere o kernel
                                             com (n+1)x(n+1)
                                             coeficientes a partir do
                                             espectro spc de um filtro
                                             com (N+1)x(N+1)
                                             pixels. */
{
  Kernel *K=NULL;


  return(K);
}

/* Questo 7 (2,0): Escolha um dos filtros R(u,v) vistos na aula sobre
   restaurao de imagens (e.g. correo de foco ou correo de
   movimento relativo entre cmera e objeto) e gere o seu espectro de
   freqncia para demonstrar o seu resultado quando aplicado a uma
   imagem degradada. OS DOIS PONTOS DESTA QUESTO INCLUEM A
   DEMONSTRAO CORRETA DOS RESULTADOS. */

Spectrum *RestaurationFilter(int ncols, int nrows)
{
  Spectrum *spc=NULL;

  return(spc);
}



